@@ -1,4 +1,4 @@
-/* Copyright (C) 2002-2024 Free Software Foundation, Inc.
+/* Copyright (C) 2002-2025 Free Software Foundation, Inc.
    Contributed by Andy Vaught
    F2003 I/O support contributed by Jerry DeLisle
 
@@ -45,7 +45,8 @@ static const char posint_required[] = "Positive integer required in format",
   bad_string[] = "Unterminated character constant in format",
   bad_hollerith[] = "Hollerith constant extends past the end of the format",
   reversion_error[] = "Exhausted data descriptors in format",
-  zero_width[] = "Zero width in format descriptor";
+  zero_width[] = "Zero width in format descriptor",
+  comma_missing[] = "Missing comma between descriptors";
 
 /* The following routines support caching format data from parsed format strings
    into a hash table.  This avoids repeatedly parsing duplicate format strings
@@ -1233,7 +1234,9 @@ parse_format_list (st_parameter_dt *dtp, bool *seen_dd)
       goto finished;
 
     default:
-      /* Assume a missing comma, this is a GNU extension */
+      /* Assume a missing comma with -std=legacy, GNU extension. */
+      if (compile_options.warn_std != 0)
+	fmt->error = comma_missing;
       goto format_item_1;
     }
 
@@ -1249,7 +1252,7 @@ parse_format_list (st_parameter_dt *dtp, bool *seen_dd)
     case FMT_RPAREN:
       goto finished;
 
-    default:			/* Assume that we have another format item */
+    default:	/* Assume that we have another format item */
       fmt->saved_token = t;
       break;
     }
@@ -1416,7 +1419,7 @@ parse_format (st_parameter_dt *dtp)
   else
     fmt->error = "Missing initial left parenthesis in format";
 
-  if (format_cache_ok)
+  if (format_cache_ok && !fmt->error)
     save_parsed_format (dtp);
   else
     dtp->u.p.format_not_saved = 1;
